Object.defineProperty(exports, '__esModule', { value: true });

var vueDemi = require('vue-demi');

function usePromiseQueue() {
    var queue = [];
    var workingOnPromise = false;
    function enqueue(promise) {
        return new Promise(function (resolve, reject) {
            queue.push({
                promise: promise,
                resolve: resolve,
                reject: reject,
            });
            dequeue();
        });
    }
    function dequeue() {
        if (workingOnPromise) {
            return false;
        }
        var item = queue.shift();
        if (!item) {
            return false;
        }
        try {
            workingOnPromise = true;
            item
                .promise()
                .then(function (value) {
                workingOnPromise = false;
                item.resolve(value);
                dequeue();
            })
                .catch(function (err) {
                workingOnPromise = false;
                item.reject(err);
                dequeue();
            });
        }
        catch (err) {
            workingOnPromise = false;
            item.reject(err);
            dequeue();
        }
        return true;
    }
    return {
        enqueue: enqueue,
        dequeue: dequeue,
    };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || from);
}

var fromEntries = function (xs) {
    return Object.fromEntries ? Object.fromEntries(xs) : xs.reduce(function (acc, _a) {
        var _b;
        var _c = __read(_a, 2), key = _c[0], value = _c[1];
        return (__assign(__assign({}, acc), (_b = {}, _b[key] = value, _b)));
    }, {});
};
function createParams(conditions, defaultParams) {
    var _conditions = __assign(__assign({}, conditions), defaultParams);
    Object.entries(_conditions).forEach(function (_a) {
        var _b = __read(_a, 2), key = _b[0], value = _b[1];
        if (Array.isArray(value)) {
            _conditions[key] = value.join(',');
        }
    });
    return _conditions;
}
function filterNoneValueObject(object) {
    return fromEntries(Object.entries(object).filter(function (item) {
        var value = item[1];
        return typeof value !== 'undefined' && value !== null && value !== '' && value.length !== 0;
    }));
}
function stringifyQuery(params, ignoreKeys) {
    var esc = encodeURIComponent;
    return Object.entries(params)
        .filter(function (_a) {
        var _b = __read(_a, 2), key = _b[0], value = _b[1];
        return typeof value !== 'undefined' &&
            value !== null &&
            value !== '' &&
            (Array.isArray(value) ? value.length !== 0 : true) &&
            (ignoreKeys && ignoreKeys.length ? !ignoreKeys.includes(key) : true);
    })
        .map(function (_a) {
        var _b = __read(_a, 2), key = _b[0], value = _b[1];
        return esc(key) + (value != null ? '=' + esc(value) : '');
    })
        .join('&');
}
function typeOf(obj) {
    return Object.prototype.toString.call(obj).slice(8, -1).toLowerCase();
}
function syncQuery2Conditions(conditions, query, backupIntiConditions) {
    var conditions2Object = __assign({}, conditions);
    var noQuery = Object.keys(query).length === 0;
    Object.keys(conditions2Object).forEach(function (key) {
        if (key in query || noQuery) {
            var conditionType = typeOf(conditions2Object[key]);
            switch (conditionType) {
                case 'date':
                    conditions[key] = noQuery ? '' : new Date(query[key]);
                    break;
                case 'array':
                    conditions[key] =
                        noQuery || !query[key].length ? [] : typeOf(query[key]) === 'string' ? query[key].split(',') : query[key];
                    if (backupIntiConditions[key].length && conditions[key].length) {
                        var originArrayValueType_1 = typeOf(backupIntiConditions[key][0]);
                        conditions[key] = conditions[key].map(function (v) {
                            switch (originArrayValueType_1) {
                                case 'number':
                                    return Number(v);
                                case 'date':
                                    return new Date(v);
                                case 'boolean':
                                    return v === 'true';
                                default:
                                    return String(v);
                            }
                        });
                    }
                    break;
                case 'string':
                    conditions[key] = noQuery ? '' : String(query[key]);
                    break;
                case 'number':
                    conditions[key] = noQuery ? 0 : Number(query[key]);
                    break;
                case 'boolean':
                    conditions[key] = noQuery ? '' : Boolean(query[key]);
                    break;
                default:
                    conditions[key] = noQuery ? '' : query[key];
                    break;
            }
        }
    });
}
function isEquivalentString(a, b, ignore) {
    return stringifyQuery(a, ignore) === stringifyQuery(b, ignore);
}
function isEquivalent(x, y) {
    if (x === null || x === undefined || y === null || y === undefined) {
        return x === y;
    }
    if (x.constructor !== y.constructor) {
        return false;
    }
    if (x instanceof Function) {
        return x === y;
    }
    if (x instanceof RegExp) {
        return x === y;
    }
    if (x === y || x.valueOf() === y.valueOf()) {
        return true;
    }
    if (Array.isArray(x) && x.length !== y.length) {
        return false;
    }
    if (x instanceof Date) {
        return false;
    }
    if (!(x instanceof Object)) {
        return false;
    }
    if (!(y instanceof Object)) {
        return false;
    }
    var p = Object.keys(x);
    return (Object.keys(y).every(function (i) {
        return p.indexOf(i) !== -1;
    }) &&
        p.every(function (i) {
            return isEquivalent(x[i], y[i]);
        }));
}
function deepClone(obj) {
    if (obj === null)
        return null;
    var clone = Object.assign({}, obj);
    Object.keys(clone).forEach(function (key) {
        if (obj[key] instanceof Date) {
            var original = obj[key];
            clone[key] = new Date(original);
            return;
        }
        clone[key] = typeof obj[key] === 'object' ? deepClone(obj[key]) : obj[key];
    });
    return Array.isArray(obj) && obj.length
        ? (clone.length = obj.length) && Array.from(clone)
        : Array.isArray(obj)
            ? Array.from(obj)
            : clone;
}
function sortObject(unordered) {
    return Object.keys(unordered)
        .sort()
        .reduce(function (obj, key) {
        obj[key] = unordered[key];
        return obj;
    }, {});
}
function serializeFunc(fn) {
    //the source code from https://github.com/yahoo/serialize-javascript/blob/main/index.js
    var serializedFn = fn.toString();
    if (/function.*?\(/.test(serializedFn)) {
        return serializedFn;
    }
    if (/.*?=>.*?/.test(serializedFn)) {
        return serializedFn;
    }
    var argsStartsAt = serializedFn.indexOf('(');
    var def = serializedFn
        .substr(0, argsStartsAt)
        .trim()
        .split(' ')
        .filter(function (val) { return val.length > 0; });
    var nonReservedSymbols = def.filter(function (val) { return ['*', 'async'].indexOf(val) === -1; });
    if (nonReservedSymbols.length > 0) {
        return ((def.indexOf('async') > -1 ? 'async ' : '') +
            'function' +
            (def.join('').indexOf('*') > -1 ? '*' : '') +
            serializedFn.substr(argsStartsAt));
    }
    return serializedFn;
}

function decode(text) {
    try {
        return decodeURIComponent('' + text);
    }
    catch (err) {
        console.error("Error decoding \"".concat(text, "\"."));
    }
    return '' + text;
}
function parseQuery(search) {
    var query = {};
    if (search === '' || search === '?')
        return query;
    var hasLeadingIM = search[0] === '?';
    var searchParams = (hasLeadingIM ? search.slice(1) : search).split('&');
    for (var i = 0; i < searchParams.length; ++i) {
        var _a = __read(searchParams[i].split('='), 2), key = _a[0], rawValue = _a[1];
        key = decode(key);
        var value = rawValue == null ? null : decode(rawValue);
        if (key in query) {
            var currentValue = query[key];
            if (!Array.isArray(currentValue)) {
                currentValue = query[key] = [currentValue];
            }
            currentValue.push(value);
        }
        else {
            query[key] = value;
        }
    }
    return query;
}
function useHistory(query, config) {
    var _this = this;
    function createQuery() {
        var href = window.location.href.split('?');
        var search = href.length === 1 ? '' : '?' + href[1];
        config.listener(parseQuery(search));
    }
    vueDemi.watch(query, function () { return __awaiter(_this, void 0, void 0, function () {
        var path, queryString, routeLocation;
        return __generator(this, function (_a) {
            path = config.sync.currentRoute.value
                ? config.sync.currentRoute.value.path
                : config.sync.currentRoute.path;
            queryString = stringifyQuery(query.value, config.ignore);
            routeLocation = path + '?' + queryString;
            try {
                config.navigation === 'replace' ? config.sync.replace(routeLocation) : config.sync.push(routeLocation);
            }
            catch (e) {
                throw new Error("[vue-condition-watcher]: history.sync is not instance of vue-router. Please check.");
            }
            return [2 /*return*/];
        });
    }); }, { deep: true });
    // initial conditions by location.search. just do once when created.
    createQuery();
    vueDemi.onMounted(function () {
        window.addEventListener('popstate', createQuery);
    });
    vueDemi.onUnmounted(function () {
        window.removeEventListener('popstate', createQuery);
    });
}

function useCache(fetcher, provider) {
    var baseKey = serializeFunc(fetcher);
    function formatString(key) {
        return "".concat(baseKey, "@").concat(stringifyQuery(sortObject(key)));
    }
    return {
        set: function (key, value) { return provider.set(formatString(key), value); },
        get: function (key) { return (provider.get(formatString(key)) ? provider.get(formatString(key)) : undefined); },
        delete: function (key) { return provider.delete(formatString(key)); },
        cached: function (key) { return provider.get(formatString(key)) !== undefined; },
    };
}

function containsProp(obj) {
    var props = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        props[_i - 1] = arguments[_i];
    }
    if (!isObject(obj))
        return false;
    return props.some(function (k) { return k in obj; });
}
var STR_UNDEFINED = 'undefined';
var hasWindow = function () { return typeof window != STR_UNDEFINED; };
var hasDocument = function () { return typeof document != STR_UNDEFINED; };
var isDocumentVisibility = function () { return hasDocument() && document.visibilityState === 'visible'; };
var hasRequestAnimationFrame = function () { return hasWindow() && typeof window['requestAnimationFrame'] != STR_UNDEFINED; };
var isNil = function (val) { return val === null || val === undefined; };
var isObject = function (val) { return val !== null && typeof val === 'object'; };
var isServer = !hasWindow();
var rAF = function (f) {
    return hasRequestAnimationFrame() ? window['requestAnimationFrame'](f) : setTimeout(f, 1);
};
var isNoData = function (data) {
    if (typeof data === 'string' || Array.isArray(data)) {
        return data.length === 0;
    }
    if ([null, undefined].includes(data)) {
        return false;
    }
    if (isObject(data)) {
        return Object.keys.length === 0;
    }
    return !data;
};

// These function inspired from VueUse's `createEventHook`
function useSubscribe() {
    var fns = [];
    var off = function (fn) {
        var index = fns.indexOf(fn);
        if (index !== -1)
            fns.splice(index, 1);
    };
    var on = function (fn) {
        fns.push(fn);
        return {
            off: function () { return off(fn); },
        };
    };
    var trigger = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        fns.forEach(function (fn) { return fn.apply(void 0, __spreadArray([], __read(args), false)); });
    };
    return {
        on: on,
        trigger: trigger,
    };
}
var online = true;
var hasWin = hasWindow();
var hasDoc = hasDocument();
function createFocusEvent(eventHook) {
    if (hasWin && window.addEventListener) {
        window.addEventListener('focus', eventHook.trigger);
    }
    return function () {
        window.removeEventListener('focus', eventHook.trigger);
    };
}
function createVisibilityEvent(eventHook) {
    if (hasDoc && document.addEventListener) {
        document.addEventListener('visibilitychange', function () { return eventHook.trigger(isDocumentVisibility()); });
    }
    return function () {
        document.removeEventListener('visibilitychange', function () { return eventHook.trigger(isDocumentVisibility()); });
    };
}
function createReconnectEvent(eventHook) {
    var onOnline = function () {
        online = true;
        eventHook.trigger(online);
    };
    // nothing to revalidate, just update the status
    var onOffline = function () {
        online = false;
        eventHook.trigger(online);
    };
    if (hasWin && window.addEventListener) {
        window.addEventListener('online', onOnline);
        window.addEventListener('offline', onOffline);
    }
    return function () {
        window.removeEventListener('online', onOnline);
        window.removeEventListener('offline', onOffline);
    };
}
function createEvents() {
    var conditionEvent = useSubscribe();
    var responseEvent = useSubscribe();
    var errorEvent = useSubscribe();
    var finallyEvent = useSubscribe();
    var focusEvent = useSubscribe();
    var visibilityEvent = useSubscribe();
    var reconnectEvent = useSubscribe();
    var stopFocusEvent = createFocusEvent(focusEvent);
    var stopVisibilityEvent = createVisibilityEvent(visibilityEvent);
    var stopReconnectEvent = createReconnectEvent(reconnectEvent);
    return {
        conditionEvent: conditionEvent,
        responseEvent: responseEvent,
        errorEvent: errorEvent,
        finallyEvent: finallyEvent,
        focusEvent: focusEvent,
        reconnectEvent: reconnectEvent,
        visibilityEvent: visibilityEvent,
        stopFocusEvent: stopFocusEvent,
        stopReconnectEvent: stopReconnectEvent,
        stopVisibilityEvent: stopVisibilityEvent,
    };
}

exports.containsProp = containsProp;
exports.createEvents = createEvents;
exports.createParams = createParams;
exports.deepClone = deepClone;
exports.filterNoneValueObject = filterNoneValueObject;
exports.hasDocument = hasDocument;
exports.hasRequestAnimationFrame = hasRequestAnimationFrame;
exports.hasWindow = hasWindow;
exports.isDocumentVisibility = isDocumentVisibility;
exports.isEquivalent = isEquivalent;
exports.isEquivalentString = isEquivalentString;
exports.isNil = isNil;
exports.isNoData = isNoData;
exports.isObject = isObject;
exports.isServer = isServer;
exports.rAF = rAF;
exports.serializeFunc = serializeFunc;
exports.sortObject = sortObject;
exports.stringifyQuery = stringifyQuery;
exports.syncQuery2Conditions = syncQuery2Conditions;
exports.typeOf = typeOf;
exports.useCache = useCache;
exports.useHistory = useHistory;
exports.usePromiseQueue = usePromiseQueue;
