Object.defineProperty(exports, '__esModule', { value: true });

var vueDemi = require('vue-demi');
var _internal = require('vue-condition-watcher/_internal');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function useConditionWatcher(config) {
    var _this = this;
    function isFetchConfig(obj) {
        return _internal.containsProp(obj, 'fetcher', 'conditions', 'defaultParams', 'initialData', 'manual', 'immediate', 'history', 'pollingInterval', 'pollingWhenHidden', 'pollingWhenOffline', 'revalidateOnFocus', 'cacheProvider', 'beforeFetch', 'afterFetch', 'onFetchError');
    }
    function isHistoryOption() {
        if (!config.history || !config.history.sync)
            return false;
        return _internal.containsProp(config.history, 'navigation', 'ignore', 'sync');
    }
    // default config
    var watcherConfig = {
        fetcher: config.fetcher,
        conditions: config.conditions,
        immediate: true,
        manual: false,
        initialData: undefined,
        pollingInterval: vueDemi.isRef(config.pollingInterval) ? config.pollingInterval : vueDemi.ref(config.pollingInterval || 0),
        pollingWhenHidden: false,
        pollingWhenOffline: false,
        revalidateOnFocus: false,
        cacheProvider: function () { return new Map(); },
    };
    // update config
    if (isFetchConfig(config)) {
        watcherConfig = __assign(__assign({}, watcherConfig), config);
    }
    var cache = _internal.useCache(watcherConfig.fetcher, watcherConfig.cacheProvider());
    var backupIntiConditions = _internal.deepClone(watcherConfig.conditions);
    var _conditions = vueDemi.reactive(watcherConfig.conditions);
    var isFetching = vueDemi.ref(false);
    var isOnline = vueDemi.ref(true);
    var isActive = vueDemi.ref(true);
    var data = vueDemi.shallowRef(cache.cached(backupIntiConditions) ? cache.get(backupIntiConditions) : watcherConfig.initialData || undefined);
    var error = vueDemi.ref(undefined);
    var query = vueDemi.ref({});
    var pollingTimer = vueDemi.ref();
    var enqueue = _internal.usePromiseQueue().enqueue;
    // - create fetch event & condition event & web event
    var _a = _internal.createEvents(), conditionEvent = _a.conditionEvent, responseEvent = _a.responseEvent, errorEvent = _a.errorEvent, finallyEvent = _a.finallyEvent, reconnectEvent = _a.reconnectEvent, focusEvent = _a.focusEvent, visibilityEvent = _a.visibilityEvent, stopFocusEvent = _a.stopFocusEvent, stopReconnectEvent = _a.stopReconnectEvent, stopVisibilityEvent = _a.stopVisibilityEvent;
    var resetConditions = function (cond) {
        Object.assign(_conditions, _internal.isObject(cond) && !cond.type ? cond : backupIntiConditions);
    };
    var isLoading = vueDemi.computed(function () { return !error.value && !data.value; });
    var conditionsChangeHandler = function (conditions, throwOnFailed) {
        if (throwOnFailed === void 0) { throwOnFailed = false; }
        return __awaiter(_this, void 0, void 0, function () {
            var checkThrowOnFailed, conditions2Object, customConditions, deepCopyCondition, isCanceled_1, validateCustomConditions, finalConditions, responseData;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        checkThrowOnFailed = typeof throwOnFailed === 'boolean' ? throwOnFailed : false;
                        if (isFetching.value)
                            return [2 /*return*/];
                        isFetching.value = true;
                        error.value = undefined;
                        conditions2Object = conditions;
                        customConditions = {};
                        deepCopyCondition = _internal.deepClone(conditions2Object);
                        if (!(typeof watcherConfig.beforeFetch === 'function')) return [3 /*break*/, 2];
                        isCanceled_1 = false;
                        return [4 /*yield*/, watcherConfig.beforeFetch(deepCopyCondition, function () {
                                isCanceled_1 = true;
                            })];
                    case 1:
                        customConditions = _a.sent();
                        if (isCanceled_1) {
                            isFetching.value = false;
                            return [2 /*return*/, Promise.resolve(undefined)];
                        }
                        if (!customConditions || typeof customConditions !== 'object' || customConditions.constructor !== Object) {
                            isFetching.value = false;
                            throw new Error("[vue-condition-watcher]: beforeFetch should return an object");
                        }
                        _a.label = 2;
                    case 2:
                        validateCustomConditions = Object.keys(customConditions).length !== 0;
                        /*
                         * if custom conditions has value, just use custom conditions
                         * filterNoneValueObject will filter no value like [] , '', null, undefined
                         * example. {name: '', items: [], age: 0, tags: null}
                         * return result will be {age: 0}
                         */
                        query.value = _internal.filterNoneValueObject(validateCustomConditions ? customConditions : conditions2Object);
                        finalConditions = _internal.createParams(query.value, watcherConfig.defaultParams);
                        responseData = undefined;
                        data.value = cache.cached(query.value) ? cache.get(query.value) : watcherConfig.initialData || undefined;
                        return [2 /*return*/, new Promise(function (resolve, reject) {
                                config
                                    .fetcher(finalConditions)
                                    .then(function (fetchResponse) { return __awaiter(_this, void 0, void 0, function () {
                                    return __generator(this, function (_a) {
                                        switch (_a.label) {
                                            case 0:
                                                responseData = fetchResponse;
                                                if (!(typeof watcherConfig.afterFetch === 'function')) return [3 /*break*/, 2];
                                                return [4 /*yield*/, watcherConfig.afterFetch(fetchResponse)];
                                            case 1:
                                                responseData = _a.sent();
                                                _a.label = 2;
                                            case 2:
                                                if (responseData === undefined) {
                                                    console.warn("[vue-condition-watcher]: \"afterFetch\" return value is ".concat(responseData, ". Please check it."));
                                                }
                                                if (!_internal.isEquivalent(data.value, responseData)) {
                                                    data.value = responseData;
                                                }
                                                if (!_internal.isEquivalent(cache.get(query.value), responseData)) {
                                                    cache.set(query.value, responseData);
                                                }
                                                responseEvent.trigger(responseData);
                                                return [2 /*return*/, resolve(fetchResponse)];
                                        }
                                    });
                                }); })
                                    .catch(function (fetchError) { return __awaiter(_this, void 0, void 0, function () {
                                    var _a;
                                    return __generator(this, function (_b) {
                                        switch (_b.label) {
                                            case 0:
                                                if (!(typeof watcherConfig.onFetchError === 'function')) return [3 /*break*/, 2];
                                                return [4 /*yield*/, watcherConfig.onFetchError({
                                                        data: undefined,
                                                        error: fetchError,
                                                    })];
                                            case 1:
                                                (_a = _b.sent(), responseData = _a.data, fetchError = _a.error);
                                                data.value = responseData || watcherConfig.initialData;
                                                error.value = fetchError;
                                                _b.label = 2;
                                            case 2:
                                                errorEvent.trigger(fetchError);
                                                if (checkThrowOnFailed) {
                                                    return [2 /*return*/, reject(fetchError)];
                                                }
                                                return [2 /*return*/, resolve(undefined)];
                                        }
                                    });
                                }); })
                                    .finally(function () {
                                    isFetching.value = false;
                                    finallyEvent.trigger();
                                    // - Start polling with out setting to manual
                                    if (watcherConfig.manual)
                                        return;
                                    polling();
                                });
                            })];
                }
            });
        });
    };
    var revalidate = function (throwOnFailed) {
        if (throwOnFailed === void 0) { throwOnFailed = false; }
        return enqueue(function () { return conditionsChangeHandler(__assign({}, _conditions), throwOnFailed); });
    };
    function execute(throwOnFailed) {
        if (throwOnFailed === void 0) { throwOnFailed = false; }
        if (_internal.isNoData(data.value) || _internal.isServer) {
            revalidate(throwOnFailed);
        }
        else {
            _internal.rAF(function () { return revalidate(throwOnFailed); });
        }
    }
    function polling() {
        if (pollingTimer.value)
            return;
        vueDemi.watchEffect(function (onCleanup) {
            if (vueDemi.unref(watcherConfig.pollingInterval)) {
                pollingTimer.value = (function () {
                    var timerId = null;
                    function next() {
                        var interval = vueDemi.unref(watcherConfig.pollingInterval);
                        if (interval && timerId !== -1) {
                            timerId = setTimeout(nun, interval);
                        }
                    }
                    function nun() {
                        // Only run when the page is visible, online and not errored.
                        if (!error.value &&
                            (watcherConfig.pollingWhenHidden || isActive.value) &&
                            (watcherConfig.pollingWhenOffline || isOnline.value)) {
                            revalidate().then(next);
                        }
                        else {
                            next();
                        }
                    }
                    next();
                    return function () { return timerId && clearTimeout(timerId); };
                })();
            }
            onCleanup(function () {
                pollingTimer.value && pollingTimer.value();
            });
        });
    }
    // - mutate: Modify `data` directly
    // - `data` is read only by default, recommend modify `data` at `afterFetch`
    // - When you need to modify `data`, you can use mutate() to directly modify data
    /*
     *  Two way to use mutate
     *  - 1.
     *     mutate(newData)
     *  - 2.
     *     mutate((currentData) => {
     *        currentData[0].name = 'runkids'
     *        return currentData
     *     })
     */
    var mutate = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var arg = args[0];
        if (arg === undefined) {
            return data.value;
        }
        if (typeof arg === 'function') {
            data.value = arg(_internal.deepClone(data.value));
        }
        else {
            data.value = arg;
        }
        cache.set(__assign({}, _conditions), data.value);
        return data.value;
    };
    // - History mode base on vue-router
    if (isHistoryOption()) {
        var historyOption = {
            sync: config.history.sync,
            ignore: config.history.ignore || [],
            navigation: config.history.navigation || 'push',
            listener: function (parsedQuery) {
                var queryObject = Object.keys(parsedQuery).length ? parsedQuery : backupIntiConditions;
                _internal.syncQuery2Conditions(_conditions, queryObject, backupIntiConditions);
            },
        };
        _internal.useHistory(query, historyOption);
    }
    // - Automatic data fetching by default
    if (!watcherConfig.manual && watcherConfig.immediate) {
        execute();
    }
    vueDemi.watch(function () { return (__assign({}, _conditions)); }, function (nc, oc) {
        // - Deep check object if be true do nothing
        if (_internal.isEquivalent(nc, oc))
            return;
        conditionEvent.trigger(_internal.deepClone(nc), _internal.deepClone(oc));
        // - Automatic data fetching until manual to be false
        !watcherConfig.manual && enqueue(function () { return conditionsChangeHandler(nc); });
    });
    reconnectEvent.on(function (status) {
        isOnline.value = status;
    });
    visibilityEvent.on(function (status) {
        isActive.value = status;
    });
    var stopSubscribeFocus = focusEvent.on(function () {
        if (!isActive.value)
            return;
        execute();
        // if (isHistoryOption() && cache.cached({ ..._conditions })) {
        //todo sync to query
        // }
    });
    if (!watcherConfig.revalidateOnFocus) {
        stopFocusEvent();
        stopSubscribeFocus.off();
    }
    if (vueDemi.getCurrentInstance()) {
        vueDemi.onUnmounted(function () {
            pollingTimer.value && pollingTimer.value();
            stopFocusEvent();
            stopReconnectEvent();
            stopVisibilityEvent();
        });
    }
    return {
        conditions: _conditions,
        data: vueDemi.readonly(data),
        error: vueDemi.readonly(error),
        isFetching: vueDemi.readonly(isFetching),
        isLoading: isLoading,
        execute: execute,
        mutate: mutate,
        resetConditions: resetConditions,
        onConditionsChange: conditionEvent.on,
        onFetchSuccess: responseEvent.on,
        onFetchError: errorEvent.on,
        onFetchFinally: finallyEvent.on,
    };
}

exports["default"] = useConditionWatcher;
